package org.zeroturnaround.javarebel.integration.support;

import java.io.IOException;

import org.zeroturnaround.javarebel.IntegrationFactory;

import java.io.InputStream;
import org.zeroturnaround.bundled.javassist.NotFoundException;

import java.net.URL;

import org.zeroturnaround.bundled.javassist.ClassPath;

import org.zeroturnaround.bundled.javassist.ByteArrayClassPath;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.LoaderClassPath;
import org.zeroturnaround.javarebel.ClassBytecodeProcessor;
import org.zeroturnaround.javarebel.LoggerFactory;

public abstract class JavassistClassBytecodeProcessor implements ClassBytecodeProcessor {
  public final byte[] process(ClassLoader cl, String classname, byte[] bytecode) {
    try {
      classname = classname.replace('/', '.');
      ClassPool cp = new ClassPool();
      cp.appendClassPath(new ByteArrayClassPath(classname, bytecode));
      cp.appendSystemPath();
      cp.appendClassPath(new LoaderClassPath(cl));
      cp.appendClassPath(new RebelClassPath(cl));
      CtClass cc = cp.get(classname);
      cc.defrost();
      process(cp, cl, cc);
      return cc.toBytecode();
    } 
    catch (Throwable e) {
      LoggerFactory.getInstance().errorEcho(e);
      throw new RuntimeException(e);
    }
  }
  
  public abstract void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception;
  
  
  private static class RebelClassPath implements ClassPath {
    private ClassLoader cl;
    
    RebelClassPath(ClassLoader cl) {
      this.cl = cl;
    }
    
    public URL find(String arg0) {
      return IntegrationFactory.getInstance().findResource(cl, arg0.replace('.', '/') + ".class");      
    }
    
    public InputStream openClassfile(String arg0) throws NotFoundException {
      try {
        URL result = find(arg0);
        return result == null ? null : result.openStream();
      } catch (IOException e) {
        LoggerFactory.getInstance().errorEcho(e);
        throw new NotFoundException(arg0, e);
      }
    }
    
    public void close() {
      // TODO Auto-generated method stub
      
    }
  }
}
